#ifndef APPLICATION_H
#define APPLICATION_H

#include <QApplication>

namespace gui {
    class SystemTray;
    class MainWindow;
}

namespace core {

class DBusWatcher;
class UPower;
class UCpufreq;

class DBusNotification;

class Application : public QApplication {
    Q_OBJECT
public:
    Application(int &argv, char *argc[]);
    virtual ~Application();

    UCpufreq * cpufreq() { return m_cpufreq; }
    UPower * upower() { return m_upower; }
    DBusWatcher * dbusWatcher() { return m_watcher; }
    core::DBusNotification * notification() { return m_notification; }
    
    static core::Application* app() {
        return (qobject_cast<core::Application*>(qApp));
    }

    void init();

private Q_SLOTS:
    void reconnectNotification();

private:
    DBusWatcher *m_watcher;
    UCpufreq *m_cpufreq;
    UPower *m_upower;
    DBusNotification *m_notification;
    gui::MainWindow *m_mainwindow;
    gui::SystemTray *m_systemtray;
};

}

#define app core::Application::app()

#endif // APPLICATION_H
