#ifndef DBUSWATCHER_H
#define DBUSWATCHER_H

#include <QObject>

#include <QDBusServiceWatcher>

namespace core {

class DBusWatcher : public QObject {
    Q_OBJECT
public:
    enum Service {
        ConsoleKit,
        UPower,
        UCpufreq,
        Notification,
        Unknown
    };

    DBusWatcher(QObject *parent = 0);

Q_SIGNALS:
    void serviceAvailable(DBusWatcher::Service);
    void serviceUnavailable(DBusWatcher::Service);

    void upowerAvailable();
    void upowerUnavailable();
    void consoleKitAvailable();
    void consoleKitUnavailable();
    void ucpufreqAvailable();
    void ucpufreqUnavailable();
    void notificationsAvailable();
    void notificationsUnavailable();

private Q_SLOTS:
    void serviceRegistered(QString service);
    void serviceUnregistered(QString service);

private:
    QDBusServiceWatcher m_system;
    QDBusServiceWatcher m_session;

    Service serviceType(const QString &service) const;

    static inline const char *upowerInterface()
    { return "org.freedesktop.UPower"; }
    static inline const char *consoleKitInterface()
    { return "org.freedesktop.ConsoleKit"; }
    static inline const char *ucpufreqInterface()
    { return "org.pyrotools.UCpufreq"; }
    static inline const char *notificationInterface()
    { return "org.freedesktop.Notifications"; }
};

}

#endif // DBUSWATCHER_H
