#ifndef BATTERY_H
#define BATTERY_H

#include <QObject>

#include <QIcon>
#include <QTime>

#include "devicedbus.h"

namespace core {
class Device : public QObject {
    Q_OBJECT
public:
    enum State {
        Unknown = 0x00,
        Charging = 0x01,
        Discharging = 0x02,
        Empty = 0x03,
        FullyCharged = 0x04,
        PendingCharge = 0x05,
        PendingDischarge = 0x06
    };

    enum Type {
        Unkown = 0x00,
        LinePower = 0x01,
        Battery = 0x02,
        Ups = 0x03,
        Monitor = 0x04,
        Mouse = 0x05,
        Keyboard = 0x06,
        Pda = 0x07,
        Phone = 0x08
    };

    enum Alert {
        Never,
        Plugged,
        Unplugged,
        Plugged_Unplugged
    };

public:
    Device(QString dbuspath, QObject *parent = 0);
    ~Device();

    const DeviceDBus & interface() const { return m_interface; }
    DeviceDBus & interface() { return m_interface; }

    Q_PROPERTY(QString Name READ name)
    QString name() const;

    Q_PROPERTY(QString PrettyName READ prettyName)
    QString prettyName() const;

    Q_PROPERTY(bool IsPowerSupply READ isPowerSupply)
    inline bool isPowerSupply() const { return m_interface.type() == 1; }

    Q_PROPERTY(QIcon Icon READ icon)
    QIcon icon() const;

    Q_PROPERTY(double LowValue READ lowValue WRITE setLowValue)
    double lowValue() const;

    Q_PROPERTY(bool AlertOnLow READ alertOnLow WRITE setEnableLowAlert)
    bool alertOnLow() const;

    Q_PROPERTY(bool AlertOnFull READ alertOnFull WRITE setEnableFullAlert)
    bool alertOnFull() const;

    Q_PROPERTY(QTime TimeLeft READ timeLeft)
    QTime timeLeft() const;

    Q_PROPERTY(bool IsPresent READ isPresent)
    bool isPresent() const;

    Q_PROPERTY(Device::Alert AlertState READ alert WRITE setAlert)
    Device::Alert alert() const;

    static Device::Alert stringToAlert(const QString &alert);

public Q_SLOTS:
    void setAlert(Device::Alert a);
    void setLowValue(double v);
    void setEnableFullAlert(bool b);
    void setEnableLowAlert(bool b);

    void refresh();

private Q_SLOTS:
    void stateChanged();

Q_SIGNALS:
    void full();
    void low();
    void changed();

    void online();
    void offline();

    void alertChanged(Device::Alert);

private:
    QSettings m_settings;
    DeviceDBus m_interface;

    bool m_low;
    bool m_full;
    bool m_online;
};
}



#endif // BATTERY_H
