#ifndef UPOWER_H
#define UPOWER_H

#include <QObject>
#include <QDBusObjectPath>
#include <QMap>

class QDBusPendingCallWatcher;

namespace core {

class Device;
class UPowerDBus;
class ConsoleKitDBus;

class UPower : public QObject {
    Q_OBJECT
public:
    UPower(QObject *parent = 0);
    virtual ~UPower();

    UPowerDBus * interface() const { return m_power; }
    UPowerDBus * upowerDBus() const { return m_power; }
    ConsoleKitDBus * consolekitDBus() const { return m_consolekit; }

    bool canSuspend();
    bool canHibernate();

    bool isUPowerPresent() const;
    bool isConsoleKitPresent() const;

    bool deviceExists(const QString &name) const;
    core::Device* device(const QString &name) const;
    inline void numberOfDevices() const { m_devices.size(); }
    const QMap<QString, core::Device*> & devices() const { return m_devices; }

    inline Device* mainDevice() const { return m_mainDevice; }

    static QString nameFromPath(const QDBusObjectPath & path);
    static QString nameFromPath(const QString &path);

public Q_SLOTS:
    bool reconnectConsoleKit();
    bool reconnectUPower();
    void setMainDevice(core::Device *d);

    bool restart();
    bool stop();
    bool hibernate();
    bool suspend();

private Q_SLOTS:
    void deviceAdded(QDBusObjectPath path);
    void deviceRemoved(QDBusObjectPath path);
    void clearDevices();

    void discoveringCompleted(QDBusPendingCallWatcher *call);

    void doSuspend();
    void doHibernate();

    void propertiesChanged();

Q_SIGNALS:
    void deviceAdded(core::Device*);
    void deviceAdded(QString);
    void deviceRemoved(QString);

    void aboutToSleep();
    void aboutToResume();
    void aboutToStop();
    void aboutToRestart();

    void mainDeviceChanged(core::Device *o, core::Device *n);
    void mainDeviceChanged(core::Device*);

    void consolekitConnected();
    void upowerConnected();
    void consolekitDisconnected();
    void upowerDisconnected();

private:
    void connectUPower();
    void connectConsoleKit();

    void addDevice(Device *b);
    void removeDevice(Device *b);
    void removeDevice(const QString &path);

    void discoverDevices();


private:
    UPowerDBus *m_power;
    ConsoleKitDBus *m_consolekit;

    QMap<QString, core::Device*> m_devices;

    Device *m_mainDevice;
};

}

#endif // UPOWER_H
