#ifndef UPOWERDBUS_H
#define UPOWERDBUS_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

namespace core {

class UPowerDBus: public QDBusAbstractInterface {
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.UPower"; }

public:
    UPowerDBus(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~UPowerDBus();

    Q_PROPERTY(bool CanHibernate READ canHibernate)
    inline bool canHibernate() const
    { return qvariant_cast< bool >(property("CanHibernate")); }

    Q_PROPERTY(bool CanSuspend READ canSuspend)
    inline bool canSuspend() const
    { return qvariant_cast< bool >(property("CanSuspend")); }

    Q_PROPERTY(QString DaemonVersion READ daemonVersion)
    inline QString daemonVersion() const
    { return qvariant_cast< QString >(property("DaemonVersion")); }

    Q_PROPERTY(bool IsDocked READ isDocked)
    inline bool isDocked() const
    { return qvariant_cast< bool >(property("IsDocked")); }

    Q_PROPERTY(bool LidForceSleep READ lidForceSleep)
    inline bool lidForceSleep() const
    { return qvariant_cast< bool >(property("LidForceSleep")); }

    Q_PROPERTY(bool LidIsClosed READ lidIsClosed)
    inline bool lidIsClosed() const
    { return qvariant_cast< bool >(property("LidIsClosed")); }

    Q_PROPERTY(bool LidIsPresent READ lidIsPresent)
    inline bool lidIsPresent() const
    { return qvariant_cast< bool >(property("LidIsPresent")); }

    Q_PROPERTY(bool OnBattery READ onBattery)
    inline bool onBattery() const
    { return qvariant_cast< bool >(property("OnBattery")); }

    Q_PROPERTY(bool OnLowBattery READ onLowBattery)
    inline bool onLowBattery() const
    { return qvariant_cast< bool >(property("OnLowBattery")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> AboutToSleep(const QString &action)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(action);
        return asyncCallWithArgumentList(QLatin1String("AboutToSleep"), argumentList);
    }

    inline QDBusPendingReply<QList<QDBusObjectPath> > EnumerateDevices()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("EnumerateDevices"), argumentList);
    }

    inline QDBusPendingReply<> Hibernate()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("Hibernate"), argumentList);
    }

    inline QDBusPendingReply<bool> HibernateAllowed()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("HibernateAllowed"), argumentList);
    }

    inline QDBusPendingReply<> Suspend()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("Suspend"), argumentList);
    }

    inline QDBusPendingReply<bool> SuspendAllowed()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("SuspendAllowed"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void Changed();
    void DeviceAdded(const QDBusObjectPath &device);
    void DeviceChanged(const QDBusObjectPath &device);
    void DeviceRemoved(const QDBusObjectPath &device);
    void NotifyResume(const QString &action);
    void NotifySleep(const QString &action);
    void Resuming();
    void Sleeping();
};


}
#endif // UPOWERDBUS_H
