#include "devicestackedwidget.h"

#include "core/device.h"
#include "gui/batterywidget.h"
#include "gui/linepowerwidget.h"
#include "core/application.h"

#include "core/upower.h"

#include <QVBoxLayout>
#include <QLabel>

using namespace gui;

DeviceStackedWidget::DeviceStackedWidget(QWidget *parent) :
    QStackedWidget(parent) {
    QWidget *w = new QWidget;
    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(new QLabel("No devices"));
    w->setLayout(vbox);
    addWidget(w);

    connect(app->upower(), SIGNAL(deviceAdded(core::Device*)), this, SLOT(addDevice(core::Device*)));
    connect(app->upower(), SIGNAL(deviceRemoved(QString)), this, SLOT(removeDevice(QString)));
    connect(this, SIGNAL(currentChanged(int)), this, SLOT(indexChanged(int)));
}

DeviceStackedWidget::~DeviceStackedWidget() {

}

void DeviceStackedWidget::addDevice(core::Device *d) {
    QWidget *w;
    switch(d->interface().type()) {
        case core::Device::Battery:
            w = new BatteryWidget(d);
            break;
        case core::Device::LinePower:
            w = new LinePowerWidget(d);
            break;
        default:
            w = new QWidget;
            break;
    }

    addWidget(w);

    if(d == app->upower()->mainDevice()) {
        setCurrentDevice(d);
    } else if(!app->upower()->mainDevice()) {
        setCurrentDevice(d);
    }
}

void DeviceStackedWidget::removeDevice(const QString &name) {
    for(int i = 0; i < count(); ++i) {
        DeviceWidget *w = qobject_cast<DeviceWidget*>(widget(i));
        if(w->device()->name() == name) {
            removeWidget(w);
            w->deleteLater();
            return;
        }
    }
}

void DeviceStackedWidget::removeDevice(core::Device *d) {
    removeDevice(d->name());
}

void DeviceStackedWidget::setCurrentDevice(core::Device *d) {
    for(int i = 1; i < count(); ++i) {
        if(device(i)->name() == d->name()) {
            setCurrentIndex(i);
            return;
        }
    }
}

void DeviceStackedWidget::noDevices() {
    setCurrentIndex(0);
}

core::Device* DeviceStackedWidget::device(int idx) const {
    DeviceWidget *w = deviceWidget(idx);
    if(w) {
        return w->device();
    }
    return 0;
}

DeviceWidget* DeviceStackedWidget::deviceWidget(int idx) const {
    return qobject_cast<DeviceWidget*>(widget(idx));
}

void DeviceStackedWidget::indexChanged(int idx) {
    core::Device *d = device(idx);
    if(d) {
        d->interface().Refresh();
    }
}
