#include "settingswidget.h"
#include "ui_settingswidget.h"

#include "core/application.h"

#include "core/upower.h"

#include <QSettings>

using namespace gui;

SettingsWidget::SettingsWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SettingsWidget)
{
    ui->setupUi(this);

    QSettings s;
    s.beginGroup("Settings");
    bool batteryOnly = s.value("ShowOnlyBatteries", false).toBool();

    ui->batteriesOnly->setChecked(batteryOnly);
    ui->mainDevice->setOnlyBattery(batteryOnly);

    ui->startMinimizedCheck->setChecked(s.value("StartMinimized", false).toBool());

    connect(ui->changeMainDevice, SIGNAL(pressed()), this, SLOT(changeMainDevice()));
    connect(ui->batteriesOnly, SIGNAL(toggled(bool)), ui->mainDevice, SLOT(setOnlyBattery(bool)));
    connect(ui->batteriesOnly, SIGNAL(toggled(bool)), this, SLOT(saveOnlyBatteries(bool)));
    connect(ui->startMinimizedCheck, SIGNAL(toggled(bool)), this, SLOT(saveStartMinimized(bool)));

    s.endGroup();
}

SettingsWidget::~SettingsWidget()
{
    delete ui;
}

void SettingsWidget::saveOnlyBatteries(bool b) {
    QSettings s;
    s.beginGroup("Settings");
    if(s.value("ShowOnlyBatteries", false).toBool() != b) {
        s.setValue("ShowOnlyBatteries", b);
        emit onlyBatteries(b);
    }
    s.endGroup();
}

void SettingsWidget::saveStartMinimized(bool b)
{
    QSettings s;
    s.beginGroup("Settings");
    if(s.value("StartMinimized", false).toBool() != b) {
        s.setValue("StartMinimized", b);
        emit startMinimized(b);
    }
    s.endGroup();
}

void SettingsWidget::changeMainDevice() {
    // Find a nicer way no to used a direct access to upower
    app->upower()->setMainDevice(ui->mainDevice->currentDevice());
}
